(function ($, Drupal, debounce) {

  Drupal.scrollama = {};

  Drupal.scrollama.getScrollamaItemsData = function($elements) {
    var elementsData = [];


    function validTimeUnit(string) {
      var match = string.match(/^((?:0.)?\d+)s?$/);

      if (match) {
        return match[1];
      } else {
        return false;
      }
    }

    $elements.each(function(){

      var $element = $(this);
      var init = $element.attr('data-scroll-init');
      var exit = $element.attr('data-scroll-exit');
      var delay = $element.attr('data-scroll-delay');

      if (!init) {
        throw new Error('You must provide at least a value for "data-scroll-init"');
      }

      var data = {
        $element: $element,
        init: init.trim().split(/\s+/),
      };

      if (exit)  {
        data.exit = exit.trim().split(/\s+/);
      };

      if (delay) {
        // Delete the "s" unit and detect wrong options.
        var filteredDelay = validTimeUnit(delay);

        if (filteredDelay) {
          data.delay = parseInt(filteredDelay, 10);
        } else {
          throw new Error('Please express your data-scroll-delay in seconds (i.e: "0.75" or "0.75s")');
        }
      } else {
        data.delay = 0;
      }

      elementsData.push(data);
    })

    return elementsData.length > 0 ? elementsData : false;

  };

  Drupal.behaviors.scrollama = {
    attach: function (context, settings) {

      $('body', context).once('scrollama').each(function(){
        var $elements = $('[data-scroll-init]');

        // Quick exit if there are no scroll-enabled elements.
        if($elements.length <= 0) {
          return;
        };

        // Get all the element's data in order to cache it in a variable
        // and improve performance.
        var elementsData = Drupal.scrollama.getScrollamaItemsData($elements);

        if (settings.scrollama.debug) {
          console.table(elementsData.map(function(item){
            return {
              item : item.$element[0],
              init: item.init.join(' '),
              exit: item.exit ? item.exit.join(' ') : false,
              delay: item.delay ? item.delay + 's' : false,
            }
          }));
        };

        var scroller = scrollama();
        scroller
          .setup({
            step: $elements.toArray(),
            offset: settings.scrollama.offset,
            debug: settings.scrollama.debug,
            once: settings.scrollama.once,
            order: settings.scrollama.order,
          })
          .onStepEnter(function(response) {
            var elementData = elementsData[response.index];
            setTimeout(function(){
              elementData.$element.addClass(elementData.init);
            }, elementData.delay * 1000)

          })
          .onStepExit(function(response) {
            var elementData = elementsData[response.index];

            if (elementData.exit) {
              setTimeout(function(){
                elementData.$element.addClass(elementData.exit);
              }, elementData.delay * 1000)
            }
          })

        $(window).on("resize", debounce(scroller.resize, 500));
      })
    }
  };
}(jQuery, Drupal, Drupal.debounce));
